<?php

namespace Teknisa\CADEST\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Teknisa\Libs\Util\Utilities;

class Ger09900AgrupamentoDeProdutosController {

	protected $dataSourceManager;
    protected $crudService;
    protected $environment;
    protected $parameterBagColumns = array();
    protected $utilities;

    public function __construct($crudService, $dataSourceManager, $environment,Utilities $utilities){
    	$this->crudService = $crudService;
        $this->dataSourceManager = $dataSourceManager;
    	$this->environment = $environment;
    	$this->utilities = $utilities;
    }

    public function saveSubGrupo($request, $response){
    	$row = $request->getDataSet()->getRows()[0];
        if($row['CDFILISUBGR']=='P'){
            $rowToSave = $this->crudService->getRowsToSave($request,'subgrupprod');
            $this->crudService->save('subgrupprod',$rowToSave);
        }
        $rowToSave = $this->crudService->getRowsToSave($request,'subgrprodfil');
        $this->crudService->save('subgrprodfil',$rowToSave);
    }

    public function saveProduto($request, $response){
        $row = $request->getDataSet()->getRows()[0];

        try{
            if($row['CDFILISUBGR']!='P'){
                if(empty($this->verificaSegMerFilial($row))){
                     throw new \Exception("Produto não cadastrado para o segmento de mercado da unidade.",1);
                }
            }
            
            if($this->verificaNiveis($row) || $this->verificaItem($row)){
                $response->setError(new Error("Produto já cadastrado para o grupo ou sub-grupo atual.",1));
            } else{
                $dataSetNV = new DataSet(null , array($row));
                $requestNV = new DTO\Request\DataSet($dataSetNV, '', '', '');
                $rowNV = $this->crudService->getRowsToSave($requestNV,'nvsubgrprod');
                $this->crudService->save('nvsubgrprod',$rowNV);
                $rows = $this->verificaProdFaltantes($row);
                if(count($rows)){
                    foreach($rows as $key => $rowToSave){
                        $rows[$key]['__is_new'] = true;
                    }
                    $dataSetPR = new DataSet(null, $rows);
                    $requestPR = new DTO\Request\DataSet($dataSetPR, '', '', '');
                    $rows = $this->crudService->getRowsToSave($requestPR,'prsubgrprod');
                    $this->crudService->save('prsubgrprod',$rows);
                }
            }
            
        } catch(\Exception $e){
            $response->setError(new Error($e->getMessage(),1));
        }

    }

    protected function verificaSegMerFilial($row){
        $conditions = array(
            array(
                "columnName" => "CDFILIAL",
                "operator" => "=",
                "value" => $row['CDFILISUBGR']
            )
        );
        $filterCriteria = new FilterCriteria('filial', $conditions);
        $filterCriteria = $this->crudService->getFilterCriteriaWithNrOrg($filterCriteria);
        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
        if($dataSet->getRows()[0]['CDSEGMER'] == null)
            return 'Nao Controla Produto Por Segmento';
            
        $this->parameterBagColumns = ['NRORG','CDSEGMER','CDARVPROD'];
        $conditions = array(
            array(
                "columnName" => "CDSEGMER",
                "operator" => "=",
                "value" => $dataSet->getRows()[0]['CDSEGMER']
            ),
            array(
                "columnName" => "CDARVPROD",
                "operator" => "=",
                "value" => $row['CDARVPROD']
            )
        );
        $filterCriteria = new FilterCriteria('produto_nivelprodseg_verifica', $conditions);
        $filterCriteria = $this->crudService->getFilterCriteriaWithNrOrg($filterCriteria);
        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria,$this->parameterBagColumns);
        $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
        return $dataSet->getRows();
    }

    protected function verificaNiveis($row){
        $this->parameterBagColumns = ['NRORG','CDGRUPPROD','CDFILISUBGR', 'CDSUBGRPROD', 'CDARVPROD'];
        $conditions = array(
            array(
                "columnName" => "CDGRUPPROD",
                "operator" => "=",
                "value" => $row['CDGRUPPROD']
            ),
            array(
                "columnName" => "CDFILISUBGR",
                "operator" => "=",
                "value" => $row['CDFILISUBGR']
            ),
            array(
                "columnName" => "CDSUBGRPROD",
                "operator" => "=",
                "value" => $row['CDSUBGRPROD']
            ),
            array(
                "columnName" => "CDARVPROD",
                "operator" => "=",
                "value" => $row['CDARVPROD']
            )
        );
        

        $filterCriteria = new FilterCriteria('produto_nvsubgrprod_verifica', $conditions);
        $filterCriteria = $this->crudService->getFilterCriteriaWithNrOrg($filterCriteria);
        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria,$this->parameterBagColumns);
        $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
        return $dataSet->getRows();
    }

    protected function verificaItem($row){
        $this->parameterBagColumns = ['NRORG','CDFILISUBGR', 'CDSUBGRPROD', 'CDGRUPPROD', 'CDARVPROD'];
        $conditions = array(
            array(
                "columnName" => "CDFILISUBGR",
                "operator" => "=",
                "value" => $row['CDFILISUBGR']
            ),
            array(
                "columnName" => "CDGRUPPROD",
                "operator" => "=",
                "value" => $row['CDGRUPPROD']
            ),
            array(
                "columnName" => "CDARVPROD",
                "operator" => "=",
                "value" => $row['CDARVPROD']
            )
        );

        $filterCriteria = new FilterCriteria('produto_nvsubgrprod_verifica_item', $conditions);
        $filterCriteria = $this->crudService->getFilterCriteriaWithNrOrg($filterCriteria);
        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria,$this->parameterBagColumns);
        $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
        return $dataSet->getRows();
    }

    protected function verificaProdFaltantes($row){
        $this->parameterBagColumns = ['NRORG','CDGRUPPROD','CDSUBGRPROD','CDFILIAL','CDNVPRODUTO','CDARVPROD'];
        $conditions = array(
            array(
                "columnName" => "CDGRUPPROD",
                "operator" => "=",
                "value" => $row['CDGRUPPROD']
            ),
            array(
                "columnName" => "CDSUBGRPROD",
                "operator" => "=",
                "value" => $row['CDSUBGRPROD']
            ),
            array(
                "columnName" => "CDFILIAL",
                "operator" => "=",
                "value" => $row['CDFILISUBGR']
            ),
            array(
                "columnName" => "CDNVPRODUTO",
                "operator" => "=",
                "value" => $row['CDNVPRODUTO']
            ),
            array(
                "columnName" => "CDARVPROD",
                "operator" => "=",
                "value" => $row['CDARVPROD']
            )
        );
        $filterCriteria = new FilterCriteria('produto_prsubgrprod', $conditions);
        $filterCriteria = $this->crudService->getFilterCriteriaWithNrOrg($filterCriteria);
        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria,$this->parameterBagColumns);
        $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
        return $dataSet->getRows();
    }

    protected function verificaSubGrupoFilial($row){
        $conditions = array(
            array(
                "columnName" => "CDGRUPPROD",
                "operator" => "=",
                "value" => $row['CDGRUPPROD']
            ),
            array(
                "columnName" => "CDSUBGRPROD",
                "operator" => "=",
                "value" => $row['CDSUBGRPROD']
            ),
            array(
                "columnName" => "CDFILISUBGR",
                "operator" => "<>",
                "value" => "P"
            )
        );
        $filterCriteria = new FilterCriteria('subgrprodfil', $conditions);
        $filterCriteria = $this->crudService->getFilterCriteriaWithNrOrg($filterCriteria);
        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria,$this->parameterBagColumns);
        $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
        return $dataSet->getRows();
    }

    public function deleteProduto($request, $response){
        $row = $request->getDataset()->getRows()[0];
        $conditions = array(
            array(
                "columnName" => "CDARVPROD",
                "operator" => "LIKE",
                "value" => $row['CDARVPROD'].'%'
            ),
            array(
                "columnName" => "CDNVPRODUTO",
                "operator" => "LIKE",
                "value" => $row['NVMAXPRODUTO']
            ),
            array(
                "columnName" => "NRORG",
                "operator" => "=",
                "value" => $this->environment->getNrOrgTrab()
                )
        );
        $filterCriteria = new FilterCriteria('produto', $conditions);
        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
        $rowsToDelete= array();
        foreach ($dataSet->getRows() as $key => $deleteRows) {
            array_push($rowsToDelete,
                array(
                    "CDGRUPPROD" => $row['CDGRUPPROD'],
                    "CDFILISUBGR" => $row['CDFILISUBGR'],
                    "CDPRODUTO" => $deleteRows['CDPRODUTO'],
                    "CDSUBGRPROD" => $row['CDSUBGRPROD'],
                    "NRORG" => $this->environment->getNrOrgTrab(),
                    "__is_new" => false
                )
            );
        }
        $dataSet = new DataSet('prsubgrprod',$rowsToDelete);
        $this->crudService->delete($response,$dataSet);
        $rowsToDelete=array();
        array_push($rowsToDelete,
                array(
                    "CDGRUPPROD" => $row['CDGRUPPROD'],
                    "CDFILISUBGR" => $row['CDFILISUBGR'],
                    "CDPRODUTO" => $row['CDPRODUTO'],
                    "CDSUBGRPROD" => $row['CDSUBGRPROD'],
                    "NRORG" => $this->environment->getNrOrgTrab(),
                    "__is_new" => false
                )
            );
        $dataSet = new DataSet('nvsubgrprod',$rowsToDelete);
        $this->crudService->delete($response,$dataSet);
        $this->utilities->saveCollectedData($row['CDOPERULTATU'],'deleteProduto','1158');
    }
    
    public function deleteGrupo($request, $response){
        $row = $request->getDataSet()->getRows()[0];
        $this->deleteSubGrupo($request, $response);
        $rowToDelete=array();
        array_push($rowToDelete,
            array(
                "CDGRUPPROD" => $row['CDGRUPPROD'],
                "NRORG" => $this->environment->getNrOrgTrab(),
                "__is_new" => false
            )
        );
        $dataSet = new DataSet('grupprod_delete', $rowToDelete);
        $this->crudService->delete($response,$dataSet);
        $this->utilities->saveCollectedData($row['CDOPERULTATU'],'deleteGrupo','1158');
    }

    public function deleteSubGrupo($request, $response){
        $row = $request->getDataSet()->getRows()[0];
        $rowToDelete=array();
        try{
            if(!empty($row['CDSUBGRPROD'])) {
                if($row['CDFILISUBGR']=='P'){
                    if(!empty($this->verificaSubGrupoFilial($row))){
                        throw new \Exception("Operação cancelada! Este sub-grupo está configurado para outra filial." ,1);
                    }
                }
            }
            $this->deleteProdutoGrupoSubGrupo($request, $response);
            if(empty($row['CDSUBGRPROD'])){
                array_push($rowToDelete,
                    array(
                        "CDGRUPPROD" => $row['CDGRUPPROD'],
                        "NRORG" => $this->environment->getNrOrgTrab(),
                        "__is_new" => false
                    )
                );
                $dataSet = new DataSet('subgrprodfil_delete_grupo', $rowToDelete);
                $this->crudService->delete($response,$dataSet);
                $dataSet = new DataSet('subgrupprod_delete_grupo', $rowToDelete);
                $this->crudService->delete($response,$dataSet);
            }else{
                array_push($rowToDelete,
                    array(
                        "CDGRUPPROD" => $row['CDGRUPPROD'],
                        "CDFILISUBGR" =>$row['CDFILISUBGR'],
                        "CDSUBGRPROD" => $row['CDSUBGRPROD'],
                        "NRORG" => $this->environment->getNrOrgTrab(),
                        "__is_new" => false
                    )
                );
                $dataSet = new DataSet('subgrprodfil_delete_subgrupo', $rowToDelete);
                $this->crudService->delete($response,$dataSet);
                if($row['CDFILISUBGR']=='P'){
                    $dataSet = new DataSet('subgrupprod_delete_subgrupo', $rowToDelete);
                    $this->crudService->delete($response,$dataSet);
                }
            }
        $this->utilities->saveCollectedData($row['CDOPERULTATU'],'deleteSubGrupo','1158');
        } catch (\Exception $e){
            $response->setError(new Error($e->getMessage(),1));
        }
    }

    protected function deleteProdutoGrupoSubGrupo($request, $response){
        $row = $request->getDataSet()->getRows()[0];
        $rowToDelete=array();
        if(empty($row['CDSUBGRPROD'])){
            array_push($rowToDelete,
                array(
                    "CDGRUPPROD" => $row['CDGRUPPROD'],
                    "NRORG" => $this->environment->getNrOrgTrab(),
                    "__is_new" => false
                )
            );
            $dataSet = new DataSet('nvsubgrprod_delete_grupo', $rowToDelete);
            $this->crudService->delete($response,$dataSet);
            $dataSet = new DataSet('prsubgrprod_delete_grupo', $rowToDelete);
            $this->crudService->delete($response,$dataSet);
        }else{
            array_push($rowToDelete,
                array(
                    "CDGRUPPROD" => $row['CDGRUPPROD'],
                    "CDFILISUBGR" =>$row['CDFILISUBGR'],
                    "CDSUBGRPROD" => $row['CDSUBGRPROD'],
                    "NRORG" => $this->environment->getNrOrgTrab(),
                    "__is_new" => false
                )
            );
            $dataSet = new DataSet('nvsubgrprod_delete_subgrupo', $rowToDelete);
            $this->crudService->delete($response,$dataSet);
            $dataSet = new DataSet('prsubgrprod_delete_subgrupo', $rowToDelete);
            $this->crudService->delete($response,$dataSet);
        }
    }
}
