<?php
namespace Teknisa\CADEST\Controllers;

use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Response\Message;
use Teknisa\CADEST\Services;

class TabelaPreco extends CrudWithParameterBag {
    
    protected $util, $parameterBag, $tabelaPrecoService;
    
    protected $dataSourceNameArray = array (
        'findPrecoProd'         => 'vnd42400_tabelapreco_precoProd',
        'findTabPrec'         => 'vnd42400_tabelapreco'
    );
    
    protected $parameterBagColumnsArray = array (
        'findPrecoProd'               =>  array('NRORG', 'CDPRODUTO'),
        'findTabPrec'               =>  array('NRORG', 'CDFILIAL'),
        'deleteTabPrec'               =>  array('NRORG', 'CDFILIAL', 'DTINIVGPREC', 'CDPRODUTO')
    );

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, Services\TabelaPreco $tabelaPrecoService) {
        parent::__construct($dataSourceManager, $parameterBag);
        $this->tabelaPrecoService = $tabelaPrecoService;
    }
    
    public function findProduct(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['findPrecoProd'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['findPrecoProd'];
        parent::find($request, $response);
    } 
    
    public function saveProdutoInline(DTO\Request $request, DTO\Response $response) {
        $row = $request->getDataSet()->getRows()[0];
        $this->dataSourceName = $this->dataSourceNameArray['findPrecoProd'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['findPrecoProd'];
        
        $priceTab = array_map(null, $row['CDFILIALTAB'], $row['CDTABEPREC'], $row['DTINIVGPREC'], $row['DTFINVGPREC']);
        foreach($priceTab as $price){
            $params = array (
                'CDPRODUTO'      => $row['CDPRODUTO'],
                'NRORG'          => $row['NRORG'],
                'CDFILIAL'       => $price[0],
                'CDTABEPREC'     => $price[1],
                'DTINIVGPREC'    => $price[2],
                'DTFINVGPREC'    => $price[3],
                'VRPRECITEM'     => $row['VRPRECITEM'],
                'CDPRPAITABPR'   => $row['CDPRPAITABPR'],
                'VRCUSTITEM'     => $row['VRCUSTITEM'],
                'VRPRECITEMCL'   => $row['VRPRECITEMCL'],
                'VRPRESUGITEM'   => $row['VRPRESUGITEM'],
                '__is_new'       => $row['__is_new']
            );
            
            $this->tabelaPrecoService->saveProdTabPrec($params);
        }
    } 
    
    public function deleteChecked(DTO\Request $request, DTO\Response $response) {
        $params = $request->getDataSet()->getRows();
        $this->tabelaPrecoService->deleteProdTabPrec($params);
    } 
    
    public function findTabPrec(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = $this->dataSourceNameArray['findTabPrec'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['findTabPrec'];
        
        $row = $request->getFilterCriteria()->getConditions();
        
        $params = array(
            ':NRORG' => $row[0]['value'],
            ':CDPRODUTO' => $row[2]['value'],
            ':CDFILIAL' => is_array($row[1]['value']) ? implode('_', $row[1]['value']) :  'T');

        $response->addDataSet(new DataSet('vnd42400_tabelapreco', $this->tabelaPrecoService->findProdTabPrec($params)));
    }
}